@@ -16,40 +16,44 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-import sys, time, datetime, re, threading
-from electrum.i18n import _, set_language
-from electrum.util import print_error, print_msg
-import os.path, json, ast, traceback
-import shutil
-import StringIO
-
-
-try:
-    import PyQt4
-except Exception:
-    sys.exit("Error: Could not import PyQt4 on Linux systems, you may try 'sudo apt-get install python-qt4'")
+import datetime
+import json
 
+import PyQt4
 from PyQt4.QtGui import *
 from PyQt4.QtCore import *
 import PyQt4.QtCore as QtCore
 
 from electrum import transaction
 from electrum.bitcoin import base_encode
+from electrum.i18n import _
 from electrum.plugins import run_hook
 
 from util import *
 
+dialogs = []  # Otherwise python randomly garbage collects the dialogs...
 
-class TxDialog(QWidget):
+def show_transaction(tx, parent, desc=None, prompt_if_unsaved=False):
+    d = TxDialog(tx, parent, desc, prompt_if_unsaved)
+    dialogs.append(d)
+    d.show()
 
-    def __init__(self, tx, parent):
+class TxDialog(QDialog):
+
+    def __init__(self, tx, parent, desc, prompt_if_unsaved):
+        '''Transactions in the wallet will show their description.
+        Pass desc to give a description for txs not yet in the wallet.
+        '''
         self.tx = tx
-        tx_dict = tx.as_dict()
+        self.tx.deserialize()
         self.parent = parent
         self.wallet = parent.wallet
-        self.saved = True
+        self.prompt_if_unsaved = prompt_if_unsaved
+        self.saved = False
+        self.broadcast = False
+        self.desc = desc
 
-        QWidget.__init__(self)
+        QDialog.__init__(self)
         self.setMinimumWidth(600)
         self.setWindowTitle(_("Transaction"))
 
@@ -65,6 +69,8 @@ class TxDialog(QWidget):
         self.status_label = QLabel()
         vbox.addWidget(self.status_label)
 
+        self.tx_desc = QLabel()
+        vbox.addWidget(self.tx_desc)
         self.date_label = QLabel()
         vbox.addWidget(self.date_label)
         self.amount_label = QLabel()
@@ -81,7 +87,6 @@ class TxDialog(QWidget):
 
         self.broadcast_button = b = QPushButton(_("Broadcast"))
         b.clicked.connect(self.do_broadcast)
-        b.hide()
 
         self.save_button = b = QPushButton(_("Save"))
         b.clicked.connect(self.save)
@@ -96,23 +101,35 @@ class TxDialog(QWidget):
 
         self.copy_button = CopyButton(lambda: str(self.tx), self.parent.app)
 
-        self.buttons = [self.copy_button, self.qr_button, self.sign_button, self.broadcast_button, self.save_button, self.cancel_button]
+        # Action buttons
+        self.buttons = [self.sign_button, self.broadcast_button, self.cancel_button]
+        # Transaction sharing buttons
+        self.sharing_buttons = [self.copy_button, self.qr_button, self.save_button]
+
         run_hook('transaction_dialog', self)
 
-        vbox.addLayout(Buttons(*self.buttons))
+        hbox = QHBoxLayout()
+        hbox.addLayout(Buttons(*self.sharing_buttons))
+        hbox.addStretch(1)
+        hbox.addLayout(Buttons(*self.buttons))
+        vbox.addLayout(hbox)
         self.update()
 
     def do_broadcast(self):
-        self.parent.broadcast_transaction(self.tx)
-        self.saved = True
+        self.parent.broadcast_transaction(self.tx, self.desc, parent=self)
+        self.broadcast = True
+        self.update()
 
-    def close(self):
-        if not self.saved:
-            if QMessageBox.question(
-                    self, _('Message'), _('This transaction is not saved. Close anyway?'),
-                    QMessageBox.Yes | QMessageBox.No, QMessageBox.No) == QMessageBox.No:
-                return
-        QWidget.close(self)
+    def closeEvent(self, event):
+        if (self.prompt_if_unsaved and not self.saved and not self.broadcast
+            and QMessageBox.question(
+                self, _('Warning'),
+                _('This transaction is not saved. Close anyway?'),
+                QMessageBox.Yes | QMessageBox.No) == QMessageBox.No):
+            event.ignore()
+        else:
+            event.accept()
+            dialogs.remove(self)
 
     def show_qr(self):
         text = self.tx.raw.decode('hex')
@@ -124,8 +141,15 @@ class TxDialog(QWidget):
 
 
     def sign(self):
-        self.parent.sign_raw_transaction(self.tx)
-        self.update()
+        def sign_done(success):
+            self.sign_button.setDisabled(False)
+            self.prompt_if_unsaved = True
+            self.saved = False
+            self.update()
+        self.sign_button.setDisabled(True)
+        cancelled, ret = self.parent.sign_tx(self.tx, sign_done, parent=self)
+        if cancelled:
+            self.sign_button.setDisabled(False)
 
 
     def save(self):
@@ -133,7 +157,7 @@ class TxDialog(QWidget):
         fileName = self.parent.getSaveFileName(_("Select where to save your signed transaction"), name, "*.txn")
         if fileName:
             with open(fileName, "w+") as f:
-                f.write(json.dumps(self.tx.as_dict(),indent=4) + '\n')
+                f.write(json.dumps(self.tx.as_dict(), indent=4) + '\n')
             self.show_message(_("Transaction saved successfully"))
             self.saved = True
 
@@ -141,34 +165,42 @@ class TxDialog(QWidget):
     def update(self):
         is_relevant, is_mine, v, fee = self.wallet.get_wallet_delta(self.tx)
         tx_hash = self.tx.hash()
-        if self.wallet.can_sign(self.tx):
-            self.sign_button.show()
-        else:
-            self.sign_button.hide()
+        desc = self.desc
+        time_str = None
+        self.broadcast_button.hide()
 
         if self.tx.is_complete():
             status = _("Signed")
 
             if tx_hash in self.wallet.transactions.keys():
+                desc, is_default = self.wallet.get_label(tx_hash)
                 conf, timestamp = self.wallet.get_confirmations(tx_hash)
                 if timestamp:
                     time_str = datetime.datetime.fromtimestamp(timestamp).isoformat(' ')[:-3]
                 else:
-                    time_str = 'pending'
+                    time_str = _('Pending')
                 status = _("%d confirmations")%conf
-                self.broadcast_button.hide()
             else:
-                time_str = None
-                conf = 0
                 self.broadcast_button.show()
+                # cannot broadcast when offline
+                if self.parent.network is None:
+                    self.broadcast_button.setEnabled(False)
         else:
             s, r = self.tx.signature_count()
             status = _("Unsigned") if s == 0 else _('Partially signed') + ' (%d/%d)'%(s,r)
-            time_str = None
-            self.broadcast_button.hide()
-            tx_hash = 'unknown'
+            tx_hash = _('Unknown');
+
+        if self.wallet.can_sign(self.tx):
+            self.sign_button.show()
+        else:
+            self.sign_button.hide()
 
         self.tx_hash_e.setText(tx_hash)
+        if desc is None:
+            self.tx_desc.hide()
+        else:
+            self.tx_desc.setText(_("Description") + ': ' + desc)
+            self.tx_desc.show()
         self.status_label.setText(_('Status:') + ' ' + status)
 
         if time_str is not None:
@@ -177,10 +209,6 @@ class TxDialog(QWidget):
         else:
             self.date_label.hide()
 
-        # cannot broadcast when offline
-        if self.parent.network is None:
-            self.broadcast_button.setEnabled(False)
-
         # if we are not synchronized, we cannot tell
         if not self.wallet.up_to_date:
             return
@@ -201,7 +229,6 @@ class TxDialog(QWidget):
         run_hook('transaction_dialog_update', self)
 
 
-
     def add_io(self, vbox):
 
         if self.tx.locktime > 0:
@@ -210,9 +237,17 @@ class TxDialog(QWidget):
         vbox.addWidget(QLabel(_("Inputs")))
 
         ext = QTextCharFormat()
-        own = QTextCharFormat()
-        own.setBackground(QBrush(QColor("lightgreen")))
-        own.setToolTip(_("Own address"))
+        rec = QTextCharFormat()
+        rec.setBackground(QBrush(QColor("lightgreen")))
+        rec.setToolTip(_("Wallet receive address"))
+        chg = QTextCharFormat()
+        chg.setBackground(QBrush(QColor("yellow")))
+        chg.setToolTip(_("Wallet change address"))
+
+        def text_format(addr):
+            if self.wallet.is_mine(addr):
+                return chg if self.wallet.is_change(addr) else rec
+            return ext
 
         i_text = QTextEdit()
         i_text.setFont(QFont(MONOSPACE_FONT))
@@ -225,8 +260,8 @@ class TxDialog(QWidget):
             else:
                 prevout_hash = x.get('prevout_hash')
                 prevout_n = x.get('prevout_n')
-                cursor.insertText(prevout_hash[0:8] + '...' + prevout_hash[-8:] + ":%d"%prevout_n, ext)
-                cursor.insertText('\t')
+                cursor.insertText(prevout_hash[0:8] + '...', ext)
+                cursor.insertText(prevout_hash[-8:] + ":%-4d " % prevout_n, ext)
                 addr = x.get('address')
                 if addr == "(pubkey)":
                     _addr = self.wallet.find_pay_to_pubkey_address(prevout_hash, prevout_n)
@@ -234,7 +269,7 @@ class TxDialog(QWidget):
                         addr = _addr
                 if addr is None:
                     addr = _('unknown')
-                cursor.insertText(addr, own if self.wallet.is_mine(addr) else ext)
+                cursor.insertText(addr, text_format(addr))
             cursor.insertBlock()
 
         vbox.addWidget(i_text)
@@ -245,10 +280,10 @@ class TxDialog(QWidget):
         o_text.setMaximumHeight(100)
         cursor = o_text.textCursor()
         for addr, v in self.tx.get_outputs():
-            cursor.insertText(addr, own if self.wallet.is_mine(addr) else ext)
+            cursor.insertText(addr, text_format(addr))
             if v is not None:
                 cursor.insertText('\t', ext)
-                cursor.insertText(self.parent.format_amount(v), ext)
+                cursor.insertText(self.parent.format_amount(v, whitespaces = True), ext)
             cursor.insertBlock()
         vbox.addWidget(o_text)
 
