@@ -20,7 +20,6 @@
 import hashlib
 import base64
 import re
-import sys
 import hmac
 
 import version
@@ -173,31 +172,6 @@ def is_old_seed(seed):
 
 # pywallet openssl private key implementation
 
-def i2d_ECPrivateKey(pkey, compressed=False):
-    if compressed:
-        key = '3081d30201010420' + \
-              '%064x' % pkey.secret + \
-              'a081a53081a2020101302c06072a8648ce3d0101022100' + \
-              '%064x' % _p + \
-              '3006040100040107042102' + \
-              '%064x' % _Gx + \
-              '022100' + \
-              '%064x' % _r + \
-              '020101a124032200'
-    else:
-        key = '308201130201010420' + \
-              '%064x' % pkey.secret + \
-              'a081a53081a2020101302c06072a8648ce3d0101022100' + \
-              '%064x' % _p + \
-              '3006040100040107044104' + \
-              '%064x' % _Gx + \
-              '%064x' % _Gy + \
-              '022100' + \
-              '%064x' % _r + \
-              '020101a144034200'
-
-    return key.decode('hex') + i2o_ECPublicKey(pkey.pubkey, compressed)
-
 def i2o_ECPublicKey(pubkey, compressed=False):
     # public keys are 65 bytes long (520 bits)
     # 0x04 + 32-byte X-coordinate + 32-byte Y-coordinate
@@ -348,10 +322,6 @@ def GetPubKey(pubkey, compressed=False):
     return i2o_ECPublicKey(pubkey, compressed)
 
 
-def GetPrivKey(pkey, compressed=False):
-    return i2d_ECPrivateKey(pkey, compressed)
-
-
 def GetSecret(pkey):
     return ('%064x' % pkey.secret).decode('hex')
 
@@ -382,11 +352,14 @@ def is_valid(addr):
 
 def is_address(addr):
     ADDRESS_RE = re.compile('[1-9A-HJ-NP-Za-km-z]{26,}\\Z')
-    if not ADDRESS_RE.match(addr): return False
+    if not ADDRESS_RE.match(addr):
+        return False
     try:
         addrtype, h = bc_address_to_hash_160(addr)
     except Exception:
         return False
+    if addrtype not in [0, 5]:
+        return False
     return addr == hash_160_to_bc_address(h, addrtype)
 
 
@@ -495,6 +468,7 @@ class MyVerifyingKey(ecdsa.VerifyingKey):
 
 
 class EC_KEY(object):
+
     def __init__( self, k ):
         secret = string_to_number(k)
         self.pubkey = ecdsa.ecdsa.Public_key( generator_secp256k1, generator_secp256k1 * secret )
@@ -504,27 +478,29 @@ class EC_KEY(object):
     def get_public_key(self, compressed=True):
         return point_to_ser(self.pubkey.point, compressed).encode('hex')
 
-    def sign_message(self, message, compressed, address):
-        private_key = ecdsa.SigningKey.from_secret_exponent( self.secret, curve = SECP256k1 )
+    def sign(self, msg_hash):
+        private_key = ecdsa.SigningKey.from_secret_exponent(self.secret, curve = SECP256k1)
         public_key = private_key.get_verifying_key()
-        signature = private_key.sign_digest_deterministic( Hash( msg_magic(message) ), hashfunc=hashlib.sha256, sigencode = ecdsa.util.sigencode_string )
-        assert public_key.verify_digest( signature, Hash( msg_magic(message) ), sigdecode = ecdsa.util.sigdecode_string)
+        signature = private_key.sign_digest_deterministic(msg_hash, hashfunc=hashlib.sha256, sigencode = ecdsa.util.sigencode_string)
+        assert public_key.verify_digest(signature, msg_hash, sigdecode = ecdsa.util.sigdecode_string)
+        return signature
+
+    def sign_message(self, message, compressed, address):
+        signature = self.sign(Hash(msg_magic(message)))
         for i in range(4):
-            sig = base64.b64encode( chr(27 + i + (4 if compressed else 0)) + signature )
+            sig = chr(27 + i + (4 if compressed else 0)) + signature
             try:
-                self.verify_message( address, sig, message)
+                self.verify_message(address, sig, message)
                 return sig
             except Exception:
                 continue
         else:
             raise Exception("error: cannot sign message")
 
-
     @classmethod
-    def verify_message(self, address, signature, message):
-        sig = base64.b64decode(signature)
-        if len(sig) != 65: raise Exception("Wrong encoding")
-
+    def verify_message(self, address, sig, message):
+        if len(sig) != 65:
+            raise Exception("Wrong encoding")
         nV = ord(sig[0])
         if nV < 27 or nV >= 35:
             raise Exception("Bad encoding")
@@ -533,16 +509,15 @@ class EC_KEY(object):
             nV -= 4
         else:
             compressed = False
-
         recid = nV - 27
-        h = Hash( msg_magic(message) )
-        public_key = MyVerifyingKey.from_signature( sig[1:], recid, h, curve = SECP256k1 )
 
+        h = Hash(msg_magic(message))
+        public_key = MyVerifyingKey.from_signature(sig[1:], recid, h, curve = SECP256k1)
         # check public key
-        public_key.verify_digest( sig[1:], h, sigdecode = ecdsa.util.sigdecode_string)
-
+        public_key.verify_digest(sig[1:], h, sigdecode = ecdsa.util.sigdecode_string)
+        pubkey = point_to_ser(public_key.pubkey.point, compressed)
         # check that we get the original signing address
-        addr = public_key_to_bc_address( point_to_ser(public_key.pubkey.point, compressed) )
+        addr = public_key_to_bc_address(pubkey)
         if address != addr:
             raise Exception("Bad signature")
 
@@ -742,6 +717,14 @@ def bip32_root(seed, testnet=False):
     return EncodeBase58Check(xprv), EncodeBase58Check(xpub)
 
 
+def xpub_from_pubkey(cK, testnet=False):
+    header_pub, header_priv = _get_headers(testnet)
+    assert cK[0] in ['\x02','\x03']
+    master_c = chr(0)*32
+    xpub = (header_pub + "00" + "00000000" + "00000000").decode("hex") + master_c + cK
+    return EncodeBase58Check(xpub)
+
+
 def bip32_private_derivation(xprv, branch, sequence, testnet=False):
     assert sequence.startswith(branch)
     if branch == sequence:
