@@ -27,7 +27,7 @@ class Blockchain():
     def __init__(self, config, network):
         self.config = config
         self.network = network
-        self.headers_url = 'http://headers.electrum.org/blockchain_headers'
+        self.headers_url = 'https://headers.electrum.org/blockchain_headers'
         self.local_height = 0
         self.set_local_height()
 
