@@ -24,6 +24,7 @@ import copy
 import argparse
 import json
 import ast
+import base64
 from functools import wraps
 from decimal import Decimal
 
@@ -34,6 +35,7 @@ from bitcoin import is_address, hash_160_to_bc_address, hash_160, COIN
 from transaction import Transaction
 import paymentrequest
 from paymentrequest import PR_PAID, PR_UNPAID, PR_UNKNOWN, PR_EXPIRED
+import contacts
 
 known_commands = {}
 
@@ -78,7 +80,7 @@ class Commands:
         self.network = network
         self._callback = callback
         self.password = None
-        self.contacts = util.Contacts(self.config)
+        self.contacts = contacts.Contacts(self.config)
 
     def _run(self, method, args, password_getter):
         cmd = known_commands[method]
@@ -199,8 +201,8 @@ class Commands:
         t = Transaction(tx)
         t.deserialize()
         if privkey:
-            pubkey = bitcoin.public_key_from_private_key(sec)
-            t.sign({pubkey:sec})
+            pubkey = bitcoin.public_key_from_private_key(privkey)
+            t.sign({pubkey:privkey})
         else:
             self.wallet.sign_transaction(t, self.password)
         return t
@@ -360,12 +362,14 @@ class Commands:
     def signmessage(self, address, message):
         """Sign a message with a key. Use quotes if your message contains
         whitespaces"""
-        return self.wallet.sign_message(address, message, self.password)
+        sig = self.wallet.sign_message(address, message, self.password)
+        return base64.b64encode(sig)
 
     @command('')
     def verifymessage(self, address, signature, message):
         """Verify a signature."""
-        return bitcoin.verify_message(address, signature, message)
+        sig = base64.b64decode(signature)
+        return bitcoin.verify_message(address, sig, message)
 
     def _mktx(self, outputs, fee, change_addr, domain, nocheck, unsigned):
         self.nocheck = nocheck
@@ -570,13 +574,29 @@ class Commands:
             else:
                 return False
         amount = int(Decimal(requested_amount)*COIN)
-        req = self.wallet.add_payment_request(addr, amount, memo, expiration, self.config)
+        expiration = int(expiration)
+        req = self.wallet.make_payment_request(addr, amount, memo, expiration)
+        self.wallet.add_payment_request(req, self.config)
         return self._format_request(req)
 
+    @command('wp')
+    def signrequest(self, address):
+        "Sign payment request with an OpenAlias"
+        alias = self.config.get('alias')
+        alias_addr = self.contacts.resolve(alias)['address']
+        self.wallet.sign_payment_request(address, alias, alias_addr, self.password)
+
     @command('w')
-    def rmrequest(self, key):
+    def rmrequest(self, address):
         """Remove a payment request"""
-        return self.wallet.remove_payment_request(key, self.config)
+        return self.wallet.remove_payment_request(address, self.config)
+
+    @command('w')
+    def clearrequests(self):
+        """Remove all payment requests"""
+        for k in self.wallet.receive_requests.keys():
+            self.wallet.remove_payment_request(k, self.config)
+
 
 param_descriptions = {
     'privkey': 'Private key. Type \'?\' to get a prompt.',
