@@ -24,6 +24,7 @@ import threading
 import traceback
 import json
 import Queue
+from collections import defaultdict
 
 import util
 from network import Network
@@ -73,7 +74,7 @@ class ClientThread(util.DaemonThread):
         self.client_pipe = util.SocketPipe(s)
         self.response_queue = Queue.Queue()
         self.server.add_client(self)
-        self.subscriptions = set()
+        self.subscriptions = defaultdict(list)
 
     def reading_thread(self):
         while self.is_running():
@@ -90,7 +91,7 @@ class ClientThread(util.DaemonThread):
                 self.server.stop()
                 continue
             if method[-10:] == '.subscribe':
-                self.subscriptions.add(repr((method, params)))
+                self.subscriptions[method].append(params)
             self.server.send_request(self, request)
 
     def run(self):
@@ -117,20 +118,16 @@ class NetworkServer(util.DaemonThread):
         util.DaemonThread.__init__(self)
         self.debug = False
         self.config = config
-        self.network = Network(config)
-        # network sends responses on that queue
-        self.network_queue = Queue.Queue()
-
-        self.running = False
+        self.pipe = util.QueuePipe()
+        self.network = Network(self.pipe, config)
         self.lock = threading.RLock()
-
         # each GUI is a client of the daemon
         self.clients = []
         self.request_id = 0
         self.requests = {}
 
     def add_client(self, client):
-        for key in ['status','banner','updated','servers','interfaces']:
+        for key in ['status', 'banner', 'updated', 'servers', 'interfaces']:
             value = self.network.get_status_value(key)
             client.response_queue.put({'method':'network.status', 'params':[key, value]})
         with self.lock:
@@ -147,18 +144,16 @@ class NetworkServer(util.DaemonThread):
             self.request_id += 1
             self.requests[self.request_id] = (request['id'], client)
             request['id'] = self.request_id
-
         if self.debug:
             print_error("-->", request)
-        self.network.requests_queue.put(request)
-
+        self.pipe.send(request)
 
     def run(self):
-        self.network.start(self.network_queue)
+        self.network.start()
         while self.is_running():
             try:
-                response = self.network_queue.get(timeout=0.1)
-            except Queue.Empty:
+                response = self.pipe.get()
+            except util.timeout:
                 continue
             if self.debug:
                 print_error("<--", response)
@@ -173,14 +168,12 @@ class NetworkServer(util.DaemonThread):
                 m = response.get('method')
                 v = response.get('params')
                 for client in self.clients:
-                    if repr((m, v)) in client.subscriptions:
+                    if m == 'network.status' or v in client.subscriptions.get(m, []):
                         client.response_queue.put(response)
-
         self.network.stop()
         print_error("server exiting")
 
 
-
 def daemon_loop(server):
     s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
     s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
@@ -214,6 +207,30 @@ def daemon_loop(server):
     print_error("Daemon exiting")
 
 
+
+def check_www_dir(rdir):
+    # rewrite index.html every time
+    import urllib, urlparse, shutil, os
+    if not os.path.exists(rdir):
+        os.mkdir(rdir)
+    index = os.path.join(rdir, 'index.html')
+    src = os.path.join(os.path.dirname(__file__), 'www', 'index.html')
+    shutil.copy(src, index)
+    files = [
+        "https://code.jquery.com/jquery-1.9.1.min.js",
+        "https://raw.githubusercontent.com/davidshimjs/qrcodejs/master/qrcode.js",
+        "https://code.jquery.com/ui/1.10.3/jquery-ui.js",
+        "https://code.jquery.com/ui/1.10.3/themes/smoothness/jquery-ui.css"
+    ]
+    for URL in files:
+        path = urlparse.urlsplit(URL).path
+        filename = os.path.basename(path)
+        path = os.path.join(rdir, filename)
+        if not os.path.exists(path):
+            print_error("downloading ", URL)
+            urllib.urlretrieve(URL, path)
+
+
 if __name__ == '__main__':
     import simple_config, util
     _config = {}
@@ -223,6 +240,12 @@ if __name__ == '__main__':
     util.set_verbosity(True)
     server = NetworkServer(config)
     server.start()
+    if config.get('websocket_server'):
+        import websockets
+        websockets.WebSocketServer(config, server).start()
+    if config.get('requests_dir'):
+        check_www_dir(config.get('requests_dir'))
+
     try:
         daemon_loop(server)
     except KeyboardInterrupt:
