@@ -7,6 +7,9 @@ import urlparse
 import urllib
 import threading
 
+def normalize_version(v):
+    return [int(x) for x in re.sub(r'(\.0+)*$','', v).split(".")]
+
 class NotEnoughFunds(Exception): pass
 
 class InvalidPassword(Exception):
@@ -45,7 +48,10 @@ class DaemonThread(threading.Thread):
             self.running = False
 
     def print_error(self, *msg):
-        print_error("[%s]"%self.__class__.__name__, *msg)
+        print_error("[%s]" % self.__class__.__name__, *msg)
+
+    def print_msg(self, *msg):
+        print_msg("[%s]" % self.__class__.__name__, *msg)
 
 
 
@@ -151,7 +157,12 @@ def age(from_date, since_date = None, target_tz=None, include_seconds=False):
     if since_date is None:
         since_date = datetime.now(target_tz)
 
-    distance_in_time = since_date - from_date
+    td = time_difference(from_date - since_date, include_seconds)
+    return td + " ago" if from_date < since_date else "in " + td
+
+
+def time_difference(distance_in_time, include_seconds):
+    #distance_in_time = since_date - from_date
     distance_in_seconds = int(round(abs(distance_in_time.days * 86400 + distance_in_time.seconds)))
     distance_in_minutes = int(round(distance_in_seconds/60))
 
@@ -159,46 +170,52 @@ def age(from_date, since_date = None, target_tz=None, include_seconds=False):
         if include_seconds:
             for remainder in [5, 10, 20]:
                 if distance_in_seconds < remainder:
-                    return "less than %s seconds ago" % remainder
+                    return "less than %s seconds" % remainder
             if distance_in_seconds < 40:
-                return "half a minute ago"
+                return "half a minute"
             elif distance_in_seconds < 60:
-                return "less than a minute ago"
+                return "less than a minute"
             else:
-                return "1 minute ago"
+                return "1 minute"
         else:
             if distance_in_minutes == 0:
-                return "less than a minute ago"
+                return "less than a minute"
             else:
-                return "1 minute ago"
+                return "1 minute"
     elif distance_in_minutes < 45:
-        return "%s minutes ago" % distance_in_minutes
+        return "%s minutes" % distance_in_minutes
     elif distance_in_minutes < 90:
-        return "about 1 hour ago"
+        return "about 1 hour"
     elif distance_in_minutes < 1440:
-        return "about %d hours ago" % (round(distance_in_minutes / 60.0))
+        return "about %d hours" % (round(distance_in_minutes / 60.0))
     elif distance_in_minutes < 2880:
-        return "1 day ago"
+        return "1 day"
     elif distance_in_minutes < 43220:
-        return "%d days ago" % (round(distance_in_minutes / 1440))
+        return "%d days" % (round(distance_in_minutes / 1440))
     elif distance_in_minutes < 86400:
-        return "about 1 month ago"
+        return "about 1 month"
     elif distance_in_minutes < 525600:
-        return "%d months ago" % (round(distance_in_minutes / 43200))
+        return "%d months" % (round(distance_in_minutes / 43200))
     elif distance_in_minutes < 1051200:
-        return "about 1 year ago"
+        return "about 1 year"
     else:
-        return "over %d years ago" % (round(distance_in_minutes / 525600))
+        return "over %d years" % (round(distance_in_minutes / 525600))
 
 block_explorer_info = {
+    'Biteasy.com': ('https://www.biteasy.com/blockchain',
+                        {'tx': 'transactions', 'addr': 'addresses'}),
+    'Bitflyer.jp': ('https://chainflyer.bitflyer.jp',
+                        {'tx': 'Transaction', 'addr': 'Address'}),
     'Blockchain.info': ('https://blockchain.info',
                         {'tx': 'tx', 'addr': 'address'}),
     'Blockr.io': ('https://btc.blockr.io',
                         {'tx': 'tx/info', 'addr': 'address/info'}),
-    'Insight.is': ('https://insight.bitpay.com',
-                        {'tx': 'tx', 'addr': 'address'}),
     'Blocktrail.com': ('https://www.blocktrail.com/BTC',
                         {'tx': 'tx', 'addr': 'address'}),
+    'Chain.so': ('https://www.chain.so',
+                        {'tx': 'tx/BTC', 'addr': 'address/BTC'}),
+    'Insight.is': ('https://insight.bitpay.com',
+                        {'tx': 'tx', 'addr': 'address'}),
     'TradeBlock.com': ('https://tradeblock.com/blockchain',
                         {'tx': 'tx', 'addr': 'address'}),
 }
@@ -229,7 +246,7 @@ def parse_URI(uri):
 
     if ':' not in uri:
         assert bitcoin.is_address(uri)
-        return uri, None, None, None, None
+        return {'address': uri}
 
     u = urlparse.urlparse(uri)
     assert u.scheme == 'bitcoin'
@@ -247,28 +264,30 @@ def parse_URI(uri):
         if len(v)!=1:
             raise Exception('Duplicate Key', k)
 
-    amount = label = message = request_url = ''
-    if 'amount' in pq:
-        am = pq['amount'][0]
+    out = {k: v[0] for k, v in pq.items()}
+    if address:
+        assert bitcoin.is_address(address)
+        out['address'] = address
+    if 'amount' in out:
+        am = out['amount']
         m = re.match('([0-9\.]+)X([0-9])', am)
         if m:
             k = int(m.group(2)) - 8
             amount = Decimal(m.group(1)) * pow(  Decimal(10) , k)
         else:
             amount = Decimal(am) * COIN
-    if 'message' in pq:
-        message = pq['message'][0].decode('utf8')
-    if 'label' in pq:
-        label = pq['label'][0]
-    if 'r' in pq:
-        request_url = pq['r'][0]
+        out['amount'] = int(amount)
+    if 'message' in out:
+        out['message'] = out['message'].decode('utf8')
+        out['memo'] = out['message']
+    if 'time' in out:
+        out['time'] = int(out['time'])
+    if 'exp' in out:
+        out['exp'] = int(out['exp'])
+    if 'sig' in out:
+        out['sig'] = bitcoin.base_decode(out['sig'], None, base=58).encode('hex')
 
-    if request_url != '':
-        return address, amount, label, message, request_url
-
-    assert bitcoin.is_address(address)
-
-    return address, amount, label, message, request_url
+    return out
 
 
 def create_URI(addr, amount, message):
@@ -461,24 +480,3 @@ class StoreDict(dict):
         if key in self.keys():
             dict.pop(self, key)
             self.save()
-
-
-import bitcoin
-from plugins import run_hook
-
-class Contacts(StoreDict):
-
-    def __init__(self, config):
-        StoreDict.__init__(self, config, 'contacts')
-
-    def resolve(self, k):
-        if bitcoin.is_address(k):
-            return {'address':k, 'type':'address'}
-        if k in self.keys():
-            _type, addr = self[k]
-            if _type == 'address':
-                return {'address':addr, 'type':'contact'}
-        out = run_hook('resolve_address', k)
-        if out:
-            return out
-        raise Exception("Invalid Bitcoin address or alias", k)
