@@ -1,21 +1,24 @@
-from PyQt4.Qt import QMessageBox, QDialog, QVBoxLayout, QLabel, QThread, SIGNAL, QGridLayout, QInputDialog, QPushButton
-import PyQt4.QtCore as QtCore
 from binascii import unhexlify
 from struct import pack
 from sys import stderr
 from time import sleep
-from base64 import b64encode, b64decode
 import unicodedata
 import threading
+import re
+
+from PyQt4.Qt import QMessageBox, QDialog, QVBoxLayout, QLabel, QThread, SIGNAL, QGridLayout, QInputDialog, QPushButton
+import PyQt4.QtCore as QtCore
 
 import electrum
+from electrum import bitcoin
+
 from electrum.account import BIP32_Account
-from electrum.bitcoin import EncodeBase58Check, public_key_to_bc_address, bc_address_to_hash_160
+from electrum.bitcoin import EncodeBase58Check, public_key_to_bc_address, bc_address_to_hash_160, xpub_from_pubkey
 from electrum.i18n import _
 from electrum.plugins import BasePlugin, hook, always_hook, run_hook
-from electrum.transaction import deserialize
+from electrum.transaction import Transaction, deserialize, is_extended_pubkey, x_to_xpub
 from electrum.wallet import BIP32_HD_Wallet
-from electrum.util import print_error
+from electrum.util import print_error, print_msg
 from electrum.wallet import pw_decode, bip32_private_derivation, bip32_root
 
 from electrum_gui.qt.util import *
@@ -30,6 +33,8 @@ try:
 except ImportError:
     TREZOR = False
 
+import trezorlib.ckd_public as ckd_public
+
 def log(msg):
     stderr.write("%s\n" % msg)
     stderr.flush()
@@ -41,19 +46,17 @@ def give_error(message):
 
 class Plugin(BasePlugin):
 
-    def fullname(self):
-        return 'Trezor Wallet'
-
-    def description(self):
-        return 'Provides support for Trezor hardware wallet\n\nRequires github.com/trezor/python-trezor'
-
     def __init__(self, config, name):
         BasePlugin.__init__(self, config, name)
         self._is_available = self._init()
         self._requires_settings = True
         self.wallet = None
-        if self._is_available:
-            electrum.wallet.wallet_types.append(('hardware', 'trezor', _("Trezor wallet"), TrezorWallet))
+        self.handler = None
+        self.client = None
+        self.transport = None
+
+    def constructor(self, s):
+        return TrezorWallet(s)
 
     def _init(self):
         return TREZOR
@@ -80,42 +83,73 @@ class Plugin(BasePlugin):
             return False
         return True
 
-    def enable(self):
-        return BasePlugin.enable(self)
+    def compare_version(self, major, minor=0, patch=0):
+        features = self.get_client().features
+        v = [features.major_version, features.minor_version, features.patch_version]
+        self.print_error('firmware version', v)
+        return cmp(v, [major, minor, patch])
 
-    def trezor_is_connected(self):
-        try:
-            self.wallet.get_client().ping('t')
-        except:
-            return False
-        return True
+    def atleast_version(self, major, minor=0, patch=0):
+        return self.compare_version(major, minor, patch) >= 0
 
-    @hook
-    def add_plugin(self, wallet):
-        wallet.plugin = self
+    def get_client(self):
+        if not TREZOR:
+            give_error('please install github.com/trezor/python-trezor')
+
+        if not self.client or self.client.bad:
+            d = HidTransport.enumerate()
+            if not d:
+                give_error('Could not connect to your Trezor. Please verify the cable is connected and that no other app is using it.')
+            self.transport = HidTransport(d[0])
+            self.client = QtGuiTrezorClient(self.transport)
+            self.client.handler = self.handler
+            self.client.set_tx_api(self)
+            self.client.bad = False
+            if not self.atleast_version(1, 2, 1):
+                self.client = None
+                give_error('Outdated Trezor firmware. Please update the firmware from https://www.mytrezor.com')
+        return self.client
 
     @hook
     def close_wallet(self):
         print_error("trezor: clear session")
-        if self.wallet and self.wallet.client:
-            self.wallet.client.clear_session()
-            self.wallet.client.transport.close()
+        if self.client:
+            self.client.clear_session()
+            self.client.transport.close()
+            self.client = None
         self.wallet = None
 
     @hook
-    def init_qt_app(self, app):
-        self.handler = TrezorQtHandler(app)
+    def cmdline_load_wallet(self, wallet):
+        self.wallet = wallet
+        self.wallet.plugin = self
+        if self.handler is None:
+            self.handler = TrezorCmdLineHandler()
 
     @hook
-    def load_wallet(self, wallet):
+    def load_wallet(self, wallet, window):
+        self.print_error("load_wallet")
         self.wallet = wallet
-        if self.trezor_is_connected():
-            if not self.wallet.check_proper_device():
-                QMessageBox.information(self.window, _('Error'), _("This wallet does not match your Trezor device"), _('OK'))
-                self.wallet.force_watching_only = True
-        else:
-            QMessageBox.information(self.window, _('Error'), _("Trezor device not detected.\nContinuing in watching-only mode."), _('OK'))
+        self.window = window
+        self.wallet.plugin = self
+
+        if self.handler is None:
+            self.handler = TrezorQtHandler(self.window.app)
+
+        try:
+            self.get_client().ping('t')
+        except BaseException as e:
+            QMessageBox.information(self.window, _('Error'), _("Trezor device not detected.\nContinuing in watching-only mode." + '\n\nReason:\n' + str(e)), _('OK'))
             self.wallet.force_watching_only = True
+            return
+
+        if self.wallet.addresses() and not self.wallet.check_proper_device():
+            QMessageBox.information(self.window, _('Error'), _("This wallet does not match your Trezor device"), _('OK'))
+            self.wallet.force_watching_only = True
+
+    @hook
+    def installwizard_load_wallet(self, wallet, window):
+        self.load_wallet(wallet, window)
 
     @hook
     def installwizard_restore(self, wizard, storage):
@@ -126,7 +160,8 @@ class Plugin(BasePlugin):
             return
         wallet = TrezorWallet(storage)
         self.wallet = wallet
-        passphrase = self.handler.get_passphrase(_("Please enter your Trezor passphrase.") + '\n' + _("Press OK if you do not use one."))
+        handler = TrezorQtHandler(wizard)
+        passphrase = handler.get_passphrase(_("Please enter your Trezor passphrase.") + '\n' + _("Press OK if you do not use one."))
         if passphrase is None:
             return
         password = wizard.password_dialog()
@@ -139,21 +174,36 @@ class Plugin(BasePlugin):
 
     @hook
     def receive_menu(self, menu, addrs):
-        if not self.wallet.is_watching_only() and self.wallet.atleast_version(1, 3) and len(addrs) == 1:
-            menu.addAction(_("Show on TREZOR"), lambda: self.wallet.show_address(addrs[0]))
+        if not self.wallet.is_watching_only() and self.atleast_version(1, 3) and len(addrs) == 1:
+            menu.addAction(_("Show on TREZOR"), lambda: self.show_address(addrs[0]))
+
+    def show_address(self, address):
+        if not self.wallet.check_proper_device():
+            give_error('Wrong device or password')
+        try:
+            address_path = self.wallet.address_id(address)
+            address_n = self.get_client().expand_path(address_path)
+        except Exception, e:
+            give_error(e)
+        try:
+            self.get_client().get_address('Bitcoin', address_n, True)
+        except Exception, e:
+            give_error(e)
+        finally:
+            self.handler.stop()
 
     def settings_widget(self, window):
         return EnterButton(_('Settings'), self.settings_dialog)
 
     def settings_dialog(self):
-        get_label = lambda: self.wallet.get_client().features.label
+        get_label = lambda: self.get_client().features.label
         update_label = lambda: current_label_label.setText("Label: %s" % get_label())
 
         d = QDialog()
         layout = QGridLayout(d)
         layout.addWidget(QLabel("Trezor Options"),0,0)
         layout.addWidget(QLabel("ID:"),1,0)
-        layout.addWidget(QLabel(" %s" % self.wallet.get_client().get_device_id()),1,1)
+        layout.addWidget(QLabel(" %s" % self.get_client().get_device_id()),1,1)
 
         def modify_label():
             response = QInputDialog().getText(None, "Set New Trezor Label", "New Trezor Label:  (upon submission confirm on Trezor)")
@@ -161,7 +211,7 @@ class Plugin(BasePlugin):
                 return
             new_label = str(response[0])
             self.handler.show_message("Please confirm label change on Trezor")
-            status = self.wallet.get_client().apply_settings(label=new_label)
+            status = self.get_client().apply_settings(label=new_label)
             self.handler.stop()
             update_label()
 
@@ -177,6 +227,128 @@ class Plugin(BasePlugin):
         else:
             return False
 
+    def sign_transaction(self, tx, prev_tx, xpub_path):
+        self.prev_tx = prev_tx
+        self.xpub_path = xpub_path
+        client = self.get_client()
+        inputs = self.tx_inputs(tx, True)
+        outputs = self.tx_outputs(tx)
+        #try:
+        signed_tx = client.sign_tx('Bitcoin', inputs, outputs)[1]
+        #except Exception, e:
+        #    give_error(e)
+        #finally:
+        self.handler.stop()
+
+        raw = signed_tx.encode('hex')
+        tx.update_signatures(raw)
+
+
+    def tx_inputs(self, tx, for_sig=False):
+        inputs = []
+        for txin in tx.inputs:
+            txinputtype = types.TxInputType()
+            if txin.get('is_coinbase'):
+                prev_hash = "\0"*32
+                prev_index = 0xffffffff # signed int -1
+            else:
+                if for_sig:
+                    x_pubkeys = txin['x_pubkeys']
+                    if len(x_pubkeys) == 1:
+                        x_pubkey = x_pubkeys[0]
+                        xpub, s = BIP32_Account.parse_xpubkey(x_pubkey)
+                        xpub_n = self.get_client().expand_path(self.xpub_path[xpub])
+                        txinputtype.address_n.extend(xpub_n + s)
+                    else:
+                        def f(x_pubkey):
+                            if is_extended_pubkey(x_pubkey):
+                                xpub, s = BIP32_Account.parse_xpubkey(x_pubkey)
+                            else:
+                                xpub = xpub_from_pubkey(x_pubkey.decode('hex'))
+                                s = []
+                            node = ckd_public.deserialize(xpub)
+                            return types.HDNodePathType(node=node, address_n=s)
+                        pubkeys = map(f, x_pubkeys)
+                        multisig = types.MultisigRedeemScriptType(
+                            pubkeys=pubkeys,
+                            signatures=map(lambda x: x.decode('hex') if x else '', txin.get('signatures')),
+                            m=txin.get('num_sig'),
+                        )
+                        txinputtype = types.TxInputType(
+                            script_type=types.SPENDMULTISIG,
+                            multisig= multisig
+                        )
+                        # find which key is mine
+                        for x_pubkey in x_pubkeys:
+                            xpub, s = BIP32_Account.parse_xpubkey(x_pubkey)
+                            if xpub in self.xpub_path:
+                                xpub_n = self.get_client().expand_path(self.xpub_path[xpub])
+                                txinputtype.address_n.extend(xpub_n + s)
+                                break
+                            else:
+                                raise
+
+                prev_hash = unhexlify(txin['prevout_hash'])
+                prev_index = txin['prevout_n']
+
+            txinputtype.prev_hash = prev_hash
+            txinputtype.prev_index = prev_index
+
+            if 'scriptSig' in txin:
+                script_sig = txin['scriptSig'].decode('hex')
+                txinputtype.script_sig = script_sig
+
+            if 'sequence' in txin:
+                sequence = txin['sequence']
+                txinputtype.sequence = sequence
+
+            inputs.append(txinputtype)
+
+        return inputs
+
+    def tx_outputs(self, tx):
+        outputs = []
+
+        for type, address, amount in tx.outputs:
+            assert type == 'address'
+            txoutputtype = types.TxOutputType()
+            if self.wallet.is_change(address):
+                address_path = self.wallet.address_id(address)
+                address_n = self.get_client().expand_path(address_path)
+                txoutputtype.address_n.extend(address_n)
+            else:
+                txoutputtype.address = address
+            txoutputtype.amount = amount
+            addrtype, hash_160 = bc_address_to_hash_160(address)
+            if addrtype == 0:
+                txoutputtype.script_type = types.PAYTOADDRESS
+            elif addrtype == 5:
+                txoutputtype.script_type = types.PAYTOSCRIPTHASH
+            else:
+                raise BaseException('addrtype')
+            outputs.append(txoutputtype)
+
+        return outputs
+
+    def electrum_tx_to_txtype(self, tx):
+        t = types.TransactionType()
+        d = deserialize(tx.raw)
+        t.version = d['version']
+        t.lock_time = d['lockTime']
+        inputs = self.tx_inputs(tx)
+        t.inputs.extend(inputs)
+        for vout in d['outputs']:
+            o = t.bin_outputs.add()
+            o.amount = vout['value']
+            o.script_pubkey = vout['scriptPubKey'].decode('hex')
+        return t
+
+    def get_tx(self, tx_hash):
+        tx = self.prev_tx[tx_hash]
+        tx.deserialize()
+        return self.electrum_tx_to_txtype(tx)
+
+
 
 
 class TrezorWallet(BIP32_HD_Wallet):
@@ -185,12 +357,10 @@ class TrezorWallet(BIP32_HD_Wallet):
 
     def __init__(self, storage):
         BIP32_HD_Wallet.__init__(self, storage)
-        self.transport = None
-        self.client = None
         self.mpk = None
         self.device_checked = False
+        self.proper_device = False
         self.force_watching_only = False
-        always_hook('add_plugin', self)
 
     def get_action(self):
         if not self.accounts:
@@ -216,32 +386,7 @@ class TrezorWallet(BIP32_HD_Wallet):
         return self.force_watching_only
 
     def get_client(self):
-        if not TREZOR:
-            give_error('please install github.com/trezor/python-trezor')
-
-        if not self.client or self.client.bad:
-            try:
-                d = HidTransport.enumerate()[0]
-                self.transport = HidTransport(d)
-            except:
-                give_error('Could not connect to your Trezor. Please verify the cable is connected and that no other app is using it.')
-            self.client = QtGuiTrezorClient(self.transport)
-            self.client.handler = self.plugin.handler
-            self.client.set_tx_api(self)
-            #self.client.clear_session()# TODO Doesn't work with firmware 1.1, returns proto.Failure
-            self.client.bad = False
-            self.device_checked = False
-            self.proper_device = False
-            if not self.atleast_version(1, 2, 1):
-                give_error('Outdated Trezor firmware. Please update the firmware from https://www.mytrezor.com')
-        return self.client
-
-    def compare_version(self, major, minor=0, patch=0):
-        features = self.get_client().features
-        return cmp([features.major_version, features.minor_version, features.patch_version], [major, minor, patch])
-
-    def atleast_version(self, major, minor=0, patch=0):
-        return self.compare_version(major, minor, patch) >= 0
+        return self.plugin.get_client()
 
     def address_id(self, address):
         account_id, (change, address_index) = self.get_address_index(address)
@@ -270,8 +415,8 @@ class TrezorWallet(BIP32_HD_Wallet):
             return xpub, None
 
     def get_public_key(self, bip32_path):
-        address_n = self.get_client().expand_path(bip32_path)
-        node = self.get_client().get_public_node(address_n).node
+        address_n = self.plugin.get_client().expand_path(bip32_path)
+        node = self.plugin.get_client().get_public_node(address_n).node
         xpub = "0488B21E".decode('hex') + chr(node.depth) + self.i4b(node.fingerprint) + self.i4b(node.child_num) + node.chain_code + node.public_key
         return EncodeBase58Check(xpub)
 
@@ -300,135 +445,51 @@ class TrezorWallet(BIP32_HD_Wallet):
         #    twd.stop()
         #return str(decrypted_msg)
 
-    def show_address(self, address):
-        if not self.check_proper_device():
-            give_error('Wrong device or password')
-        try:
-            address_path = self.address_id(address)
-            address_n = self.get_client().expand_path(address_path)
-        except Exception, e:
-            give_error(e)
-        try:
-            self.get_client().get_address('Bitcoin', address_n, True)
-        except Exception, e:
-            give_error(e)
-        finally:
-            self.plugin.handler.stop()
-
     def sign_message(self, address, message, password):
         if not self.check_proper_device():
             give_error('Wrong device or password')
         try:
             address_path = self.address_id(address)
-            address_n = self.get_client().expand_path(address_path)
+            address_n = self.plugin.get_client().expand_path(address_path)
         except Exception, e:
             give_error(e)
         try:
-            msg_sig = self.get_client().sign_message('Bitcoin', address_n, message)
+            msg_sig = self.plugin.get_client().sign_message('Bitcoin', address_n, message)
         except Exception, e:
             give_error(e)
         finally:
             self.plugin.handler.stop()
-        b64_msg_sig = b64encode(msg_sig.signature)
-        return str(b64_msg_sig)
+        return msg_sig.signature
 
     def sign_transaction(self, tx, password):
         if tx.is_complete():
             return
         if not self.check_proper_device():
             give_error('Wrong device or password')
-        client = self.get_client()
-        inputs = self.tx_inputs(tx)
-        outputs = self.tx_outputs(tx)
-        try:
-            signed_tx = self.get_client().sign_tx('Bitcoin', inputs, outputs)[1]
-        except Exception, e:
-            give_error(e)
-        finally:
-            self.plugin.handler.stop()
-        #values = [i['value'] for i in tx.inputs]
-        raw = signed_tx.encode('hex')
-        tx.update(raw)
-        #for i, txinput in enumerate(tx.inputs):
-        #    txinput['value'] = values[i]
+        # previous transactions used as inputs
+        prev_tx = {}
+        # path of the xpubs that are involved
+        xpub_path = {}
+        for txin in tx.inputs:
+            tx_hash = txin['prevout_hash']
 
-    def tx_inputs(self, tx):
-        inputs = []
-        for txinput in tx.inputs:
-            txinputtype = types.TxInputType()
-            if ('is_coinbase' in txinput and txinput['is_coinbase']):
-                prev_hash = "\0"*32
-                prev_index = 0xffffffff # signed int -1
-            else:
-                address = txinput['address']
-                try:
-                    address_path = self.address_id(address)
-                    address_n = self.get_client().expand_path(address_path)
-                    txinputtype.address_n.extend(address_n)
-                except: pass
+            ptx = self.transactions.get(tx_hash)
+            if ptx is None:
+                ptx = self.network.synchronous_get([('blockchain.transaction.get', [tx_hash])])[0]
+                ptx = Transaction(ptx)
+            prev_tx[tx_hash] = ptx
 
-                prev_hash = unhexlify(txinput['prevout_hash'])
-                prev_index = txinput['prevout_n']
+            for x_pubkey in txin['x_pubkeys']:
+                if not is_extended_pubkey(x_pubkey):
+                    continue
+                xpub = x_to_xpub(x_pubkey)
+                for k, v in self.master_public_keys.items():
+                    if v == xpub:
+                        account_id = re.match("x/(\d+)'", k).group(1)
+                        account_derivation = "44'/0'/%s'"%account_id
+                xpub_path[xpub] = account_derivation
 
-            txinputtype.prev_hash = prev_hash
-            txinputtype.prev_index = prev_index
-
-            if 'scriptSig' in txinput:
-                script_sig = txinput['scriptSig'].decode('hex')
-                txinputtype.script_sig = script_sig
-
-            if 'sequence' in txinput:
-                sequence = txinput['sequence']
-                txinputtype.sequence = sequence
-
-            inputs.append(txinputtype)
-            #TODO P2SH
-        return inputs
-
-    def tx_outputs(self, tx):
-        outputs = []
-
-        for type, address, amount in tx.outputs:
-            assert type == 'address'
-            txoutputtype = types.TxOutputType()
-            if self.is_change(address):
-                address_path = self.address_id(address)
-                address_n = self.get_client().expand_path(address_path)
-                txoutputtype.address_n.extend(address_n)
-            else:
-                txoutputtype.address = address
-            txoutputtype.amount = amount
-            addrtype, hash_160 = bc_address_to_hash_160(address)
-            if addrtype == 0:
-                txoutputtype.script_type = types.PAYTOADDRESS
-            elif addrtype == 5:
-                txoutputtype.script_type = types.PAYTOSCRIPTHASH
-            else:
-                raise BaseException('addrtype')
-            outputs.append(txoutputtype)
-
-        return outputs
-
-    def electrum_tx_to_txtype(self, tx):
-        t = types.TransactionType()
-        d = deserialize(tx.raw)
-        t.version = d['version']
-        t.lock_time = d['lockTime']
-
-        inputs = self.tx_inputs(tx)
-        t.inputs.extend(inputs)
-
-        for vout in d['outputs']:
-            o = t.bin_outputs.add()
-            o.amount = vout['value']
-            o.script_pubkey = vout['scriptPubKey'].decode('hex')
-
-        return t
-
-    def get_tx(self, tx_hash):
-        tx = self.transactions[tx_hash]
-        tx.deserialize()
-        return self.electrum_tx_to_txtype(tx)
+        self.plugin.sign_transaction(tx, prev_tx, xpub_path)
 
     def check_proper_device(self):
         self.get_client().ping('t')
@@ -494,6 +555,27 @@ class TrezorGuiMixin(object):
         return proto.WordAck(word=word)
 
 
+class TrezorCmdLineHandler:
+
+    def get_passphrase(self, msg):
+        import getpass
+        print_msg(msg)
+        return getpass.getpass('')
+
+    def get_pin(self, msg):
+        t = { 'a':'7', 'b':'8', 'c':'9', 'd':'4', 'e':'5', 'f':'6', 'g':'1', 'h':'2', 'i':'3'}
+        print_msg(msg)
+        print_msg("a b c\nd e f\ng h i\n-----")
+        o = raw_input()
+        return ''.join(map(lambda x: t[x], o))
+
+    def stop(self):
+        pass
+
+    def show_message(self, msg):
+        print_msg(msg)
+
+
 class TrezorQtHandler:
 
     def __init__(self, win):
