@@ -34,7 +34,7 @@ from electrum import bitcoin
 from electrum.bitcoin import *
 from electrum.mnemonic import Mnemonic
 from electrum import version
-from electrum.wallet import Wallet_2of3
+from electrum.wallet import Multisig_Wallet, BIP32_Wallet
 from electrum.i18n import _
 from electrum.plugins import BasePlugin, run_hook, hook
 
@@ -62,7 +62,7 @@ class TrustedCoinCosignerClient(object):
         self.base_url = base_url
         self.debug = debug
         self.user_agent = user_agent
-        
+
     def send_request(self, method, relative_url, data=None):
         kwargs = {'headers': {}}
         if self.user_agent:
@@ -85,12 +85,12 @@ class TrustedCoinCosignerClient(object):
                 r = response.json()
                 if 'message' in r:
                     message = r['message']
-            raise TrustedCoinException(message, response.status_code)        
+            raise TrustedCoinException(message, response.status_code)
         if response.headers.get('content-type') == 'application/json':
             return response.json()
         else:
             return response.text
-        
+
     def get_terms_of_service(self, billing_plan='electrum-per-tx-otp'):
         """
         Returns the TOS for the given billing plan as a plain/text unicode string.
@@ -98,13 +98,13 @@ class TrustedCoinCosignerClient(object):
         """
         payload = {'billing_plan': billing_plan}
         return self.send_request('get', 'tos', payload)
-        
+
     def create(self, xpubkey1, xpubkey2, email, billing_plan='electrum-per-tx-otp'):
         """
         Creates a new cosigner resource.
         :param xpubkey1: a bip32 extended public key (customarily the hot key)
         :param xpubkey2: a bip32 extended public key (customarily the cold key)
-        :param email: a contact email 
+        :param email: a contact email
         :param billing_plan: the billing plan for the cosigner
         """
         payload = {
@@ -114,7 +114,7 @@ class TrustedCoinCosignerClient(object):
             'billing_plan': billing_plan,
         }
         return self.send_request('post', 'cosigner', payload)
-        
+
     def auth(self, id, otp):
         """
         Attempt to authenticate for a particular cosigner.
@@ -123,7 +123,7 @@ class TrustedCoinCosignerClient(object):
         """
         payload = {'otp': otp}
         return self.send_request('post', 'cosigner/%s/auth' % quote(id), payload)
-        
+
     def get(self, id):
         """
         Attempt to authenticate for a particular cosigner.
@@ -131,7 +131,7 @@ class TrustedCoinCosignerClient(object):
         :param otp: the one time password
         """
         return self.send_request('get', 'cosigner/%s' % quote(id))
-        
+
     def sign(self, id, transaction, otp):
         """
         Attempt to authenticate for a particular cosigner.
@@ -157,7 +157,7 @@ class TrustedCoinCosignerClient(object):
             'otp': otp,
             'recipient': recipient,
             'timestamp': int(time.time()),
-            
+
         }
         relative_url = 'cosigner/%s/transfer' % quote(id)
         full_url = urljoin(self.base_url, relative_url)
@@ -170,9 +170,13 @@ class TrustedCoinCosignerClient(object):
 server = TrustedCoinCosignerClient(user_agent="Electrum/" + version.ELECTRUM_VERSION)
 
 
-class Wallet_2fa(Wallet_2of3):
+class Wallet_2fa(Multisig_Wallet):
 
-    wallet_type = '2fa'
+    def __init__(self, storage):
+        BIP32_Wallet.__init__(self, storage)
+        self.wallet_type = '2fa'
+        self.m = 2
+        self.n = 3
 
     def get_action(self):
         xpub1 = self.master_public_keys.get("x1/")
@@ -191,13 +195,13 @@ class Wallet_2fa(Wallet_2of3):
         return Mnemonic('english').make_seed(num_bits=256, prefix=SEED_PREFIX)
 
     def estimated_fee(self, tx):
-        fee = Wallet_2of3.estimated_fee(self, tx)
+        fee = Multisig_Wallet.estimated_fee(self, tx)
         x = run_hook('extra_fee', tx)
         if x: fee += x
         return fee
 
     def get_tx_fee(self, tx):
-        fee = Wallet_2of3.get_tx_fee(self, tx)
+        fee = Multisig_Wallet.get_tx_fee(self, tx)
         x = run_hook('extra_fee', tx)
         if x: fee += x
         return fee
@@ -210,17 +214,12 @@ class Plugin(BasePlugin):
 
     def __init__(self, x, y):
         BasePlugin.__init__(self, x, y)
-        electrum.wallet.wallet_types.append(('twofactor', '2fa', _("Wallet with two-factor authentication"), Wallet_2fa))
         self.seed_func = lambda x: bitcoin.is_new_seed(x, SEED_PREFIX)
         self.billing_info = None
         self.is_billing = False
 
-    def fullname(self):
-        return 'Two Factor Authentication'
-
-    def description(self):
-        return _("This plugin adds two-factor authentication to your wallet.") + '<br/>'\
-            + _("For more information, visit") + " <a href=\"https://api.trustedcoin.com/#/electrum-help\">https://api.trustedcoin.com/#/electrum-help</a>"
+    def constructor(self, s):
+        return Wallet_2fa(s)
 
     def is_available(self):
         if not self.wallet:
@@ -266,13 +265,6 @@ class Plugin(BasePlugin):
         address = public_key_to_bc_address( cK )
         return address
 
-    def enable(self):
-        if self.is_enabled():
-            self.window.show_message('Error: Two-factor authentication is already activated on this wallet')
-            return
-        self.set_enabled(True)
-        self.window.show_message('Two-factor authentication is enabled.')
-
     def create_extended_seed(self, wallet, window):
         seed = wallet.make_seed()
         if not window.show_seed(seed, None):
@@ -290,7 +282,7 @@ class Plugin(BasePlugin):
         wallet.add_cosigner_seed(' '.join(words[0:n]), 'x1/', password)
         wallet.add_cosigner_xpub(' '.join(words[n:]), 'x2/')
 
-        msg = [ 
+        msg = [
             _('Your wallet file is:') + " %s"%os.path.abspath(wallet.storage.path),
             _('You need to be online in order to complete the creation of your wallet.'),
             _('If you generated your seed on an offline computer, click on "%s" to close this window, move your wallet file to an online computer and reopen it with Electrum.') % _('Close'),
@@ -334,7 +326,9 @@ class Plugin(BasePlugin):
         self.is_billing = False
 
     @hook
-    def load_wallet(self, wallet):
+    def load_wallet(self, wallet, window):
+        self.wallet = wallet
+        self.window = window
         self.trustedcoin_button = StatusBarButton( QIcon(":icons/trustedcoin.png"), _("Network"), self.settings_dialog)
         self.window.statusBar().addPermanentWidget(self.trustedcoin_button)
         self.xpub = self.wallet.master_public_keys.get('x1/')
@@ -343,6 +337,11 @@ class Plugin(BasePlugin):
         t.setDaemon(True)
         t.start()
 
+    @hook
+    def installwizard_load_wallet(self, wallet, window):
+        self.wallet = wallet
+        self.window = window
+
     @hook
     def close_wallet(self):
         self.window.statusBar().removeWidget(self.trustedcoin_button)
@@ -351,14 +350,14 @@ class Plugin(BasePlugin):
     def get_wizard_action(self, window, wallet, action):
         if hasattr(self, action):
             return getattr(self, action)
-            
+
     @hook
     def installwizard_restore(self, window, storage):
-        if storage.get('wallet_type') != '2fa': 
+        if storage.get('wallet_type') != '2fa':
             return
 
         seed = window.enter_seed_dialog("Enter your seed", None, func=self.seed_func)
-        if not seed: 
+        if not seed:
             return
         wallet = Wallet_2fa(storage)
         self.wallet = wallet
@@ -381,7 +380,7 @@ class Plugin(BasePlugin):
         self.wallet = wallet
         self.window = window
 
-        if wallet.storage.get('wallet_type') != '2fa': 
+        if wallet.storage.get('wallet_type') != '2fa':
             raise
             return
 
@@ -445,8 +444,8 @@ class Plugin(BasePlugin):
         return False
 
     @hook
-    def send_tx(self, tx):
-        self.print_error("twofactor:send_tx")
+    def sign_tx(self, tx):
+        self.print_error("twofactor:sign_tx")
         if self.wallet.storage.get('wallet_type') != '2fa':
             return
 
@@ -515,7 +514,7 @@ class Plugin(BasePlugin):
 
         self.print_error( "received answer", r)
         if not r:
-            return 
+            return
 
         raw_tx = r.get('transaction')
         tx.update(raw_tx)
@@ -535,7 +534,7 @@ class Plugin(BasePlugin):
         grid.addWidget(pw, 1, 1)
         vbox.addLayout(grid)
         vbox.addLayout(Buttons(CancelButton(d), OkButton(d)))
-        if not d.exec_(): 
+        if not d.exec_():
             return
         return pw.get_amount()
 
@@ -563,7 +562,7 @@ class Plugin(BasePlugin):
               + _("For more information, visit") + " <a href=\"https://api.trustedcoin.com/#/electrum-help\">https://api.trustedcoin.com/#/electrum-help</a>"
         label = QLabel(msg)
         label.setOpenExternalLinks(1)
-        
+
         hbox.addStretch(10)
         hbox.addWidget(logo)
         hbox.addStretch(10)
@@ -657,7 +656,7 @@ class Plugin(BasePlugin):
             tos = server.get_terms_of_service()
             self.TOS = tos
             window.emit(SIGNAL('twofactor:TOS'))
-            
+
         def on_result():
             tos_e.setText(self.TOS)
 
@@ -670,7 +669,7 @@ class Plugin(BasePlugin):
         email_e.textChanged.connect(lambda: accept_button.setEnabled(re.match(regexp,email_e.text()) is not None))
         email_e.setFocus(True)
 
-        if not window.exec_(): 
+        if not window.exec_():
             return
 
         email = str(email_e.text())
